==> views_string_aggregation-1.0.2.zip: views_string_aggregation/README.md <==
# Views String Aggregation

A Drupal module that provides database-specific SQL string aggregation support for Views, enabling efficient concatenation of multiple field values into a single aggregated result.

## Features

- **Database-Specific Implementation**: Supports MySQL/MariaDB `GROUP_CONCAT`, SQLite `GROUP_CONCAT` and PostgreSQL `STRING_AGG` functions
- **SQL-Level Aggregation**: Performs concatenation at the database level for optimal performance
- **Configurable Separators**: Customize the separator used between concatenated values
- **Sorting Support**: Order concatenated values with support for custom ordering
- **Views Integration**: Automatically extends Drupal Views with minimal configuration
- **Exposed Filter Support**: Works with Views exposed filters and grouping
- **Distinct Values**: Supports distinct value aggregation
- **Maximum length**: Supports configuring maximum length via group_concat_max_len (MySQL/MariaDB only)

## Requirements

- Drupal 10.x or 11.x
- Views module (core)
- Supported databases:
  - MySQL 5.7+ / MariaDB 10.2+
  - PostgreSQL 9.1+
  - SQLite 3+

## Installation

### Composer (Recommended)

```
composer require drupal/views_string_aggregation
```

### Manual (Drush)

1. Download and place the module in your `modules/custom/` directory
2. Enable the module: `drush en views_string_aggregation`
3. Clear cache: `drush cr`

### Manual (UI)

1. Download and place the module in your `modules/custom/` directory
2. Enable the module at Admin > Modules

## Usage

### Basic Setup

1. Create or edit a View
2. Enable **"Use aggregation"** in the view's **Advanced** settings
3. Add the field you want to aggregate
4. Set the field's **Aggregation type** to **"String aggregation"** or **"String aggregation DISTINCT"**

### Configuration Options

#### Field Configuration
Set fields to use string aggregation by selecting **"String aggregation"** or **"String aggregation DISTINCT"** as the aggregation type in the field configuration.

#### Query Settings
When aggregation is enabled, additional setting options become available, under Views **"Advanced > Other > Query settings"**:

- **Concatenated values separator**: `SEPARATOR` character(s) to separate aggregated values (default: `,`)
- **Order concatenated values by**: Select an `ORDER BY` field to order concatenated values
- **Order direction**: Ascending or Descending sort order for concatenated values
- **Concatenated values maximum length**: Maximum length via group_concat_max_len (MySQL/MariaDB only)

### Example Use Cases

#### 1. Node Tags Aggregation
Group nodes by category and show all tags as a comma-separated list:
```
Content Type | Tags
-------------|------------------
Article      | drupal, cms, web
Blog         | personal, tech
```

#### 2. User Roles Summary
Display all roles for users grouped by department:
```
Department | User Roles
-----------|------------------------
IT         | administrator, editor
Marketing  | editor, content creator
```

### Database-Specific Features

#### MySQL/MariaDB
- Uses `GROUP_CONCAT` for standard aggregation
- Uses `GROUP_CONCAT(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within `GROUP_CONCAT`
- Configurable separator and ordering
- Optionally override default max length via `group_concat_max_len`

#### PostgreSQL
- Uses `STRING_AGG` for standard aggregation
- Uses `STRING_AGG(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within aggregation function
- Configurable separator and ordering
- Note: order by field supports ASC/DESC direction
  but must use the aggregated field when using `DISTINCT` with pgsql

#### SQLite
- Uses `GROUP_CONCAT` for standard aggregation
- Uses `GROUP_CONCAT(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within `GROUP_CONCAT`
- Configurable separator and ordering
- Note: custom separator is not supported when using `DISTINCT` with SQLite

## API Reference

### Hook Implementations

- `hook_views_plugins_query_alter()`: Automatically extends the default Views query plugin
- `hook_config_schema_info_alter()`: Extends Views `views_query` schema with custom configuration options

### Query Plugin Classes

- `VsaBase`: Base class extending Drupal's SQL query plugin
- `MySql`: MySQL/MariaDB-specific implementation with `GROUP_CONCAT`
- `PgSql`: PostgreSQL-specific implementation with `STRING_AGG`
- `Sqlite`: SQLite-specific implementation with `GROUP_CONCAT`

### Filter Handlers

- `GroupByString`: Specialized filter for grouped string fields using `HAVING` clauses

### Contextual Filter Handlers

- `GroupByString`: Specialized contextual filter argument for grouped string fields using `HAVING` clauses

## Advanced Configuration

### Disable Automatic Query Plugin Extension
If you prefer to disable the automatic extension of the views_query plugin,
you can do so and then manually control which views use string aggregation:

1. Set `extend_views_query_plugin: false` in configuration
2. Clear cache
3. Manually assign Views String Aggregation query plugins to a table in `hook_views_data()`:

```php
// mysql
$views_data['table']['base']['query_id'] = 'vsa_views_query_mysql';
// postgres
$views_data['table']['base']['query_id'] = 'vsa_views_query_pgsql';
// or sqlite
$views_data['table']['base']['query_id'] = 'vsa_views_query_sqlite';
```

## Troubleshooting

**String aggregation option not appearing**
- Ensure "Use aggregation" is enabled in View advanced settings
- Verify the module is enabled and cache is cleared

### Performance Considerations

- String aggregation is performed at the SQL level for optimal performance
- Consider using `DISTINCT` aggregation when appropriate to reduce duplicate data
- Large result sets with many concatenated values may impact query performance

## Testing

The module includes automated kernel tests covering:
- Basic string aggregation functionality
- Distinct string aggregation functionality
- Separator configuration
- Exposed filter integration
- Sort order validation
- Database-specific implementations
- Maximum length configuration (MySQL/MariaDB only)

Run tests with:
```bash
./vendor/bin/phpunit modules/custom/views_string_aggregation/tests/
```

## Credits

This module acknowledges with thanks the initial work contributed in
<https://www.drupal.org/project/drupal/issues/2902481> namely by:

<!-- cspell:disable -->

- f2boot
- phjou
- navkaur
- kapilv
- jackson.cooper
- attisan
- simeonkesmev
- achton
- kasperg
- Sir-Arturio
- b.ravanbakhsh

<!-- cspell:enable -->

## Contributing

This module follows Drupal coding standards and best practices. Contributions are welcome for:
- Additional database engine support
- Performance optimizations
- Enhanced configuration options
- Bug fixes and improvements

## License

This project is licensed under the GPL-2.0+ license.

## Support

For issues, feature requests, or questions:
1. Check existing issues in the project repository
2. Review the troubleshooting section above
3. Create a detailed issue report with:
   - Drupal version
   - Database type and version
   - Steps to reproduce
   - Expected vs actual behavior

==> views_string_aggregation-9.0.0.zip: views_string_aggregation/README.md <==
# Views String Aggregation

A Drupal module that provides database-specific SQL string aggregation support for Views, enabling efficient concatenation of multiple field values into a single aggregated result.

## Features

- **Database-Specific Implementation**: Supports MySQL/MariaDB `GROUP_CONCAT`, SQLite `GROUP_CONCAT` and PostgreSQL `STRING_AGG` functions
- **SQL-Level Aggregation**: Performs concatenation at the database level for optimal performance
- **Configurable Separators**: Customize the separator used between concatenated values
- **Sorting Support**: Order concatenated values with support for custom ordering
- **Views Integration**: Automatically extends Drupal Views with minimal configuration
- **Exposed Filter Support**: Works with Views exposed filters and grouping
- **Distinct Values**: Supports distinct value aggregation
- **Maximum length**: Supports configuring maximum length via group_concat_max_len (MySQL/MariaDB)

## Requirements

- Drupal 9.x
- PHP >= 8.1
- Views module (core)
- Supported databases:
  - MySQL 5.7+ / MariaDB 10.2+
  - PostgreSQL 9.1+
  - SQLite 3+

## Installation

### Composer (Recommended)

```
composer require drupal/views_string_aggregation
```

### Manual (Drush)

1. Download and place the module in your `modules/custom/` directory
2. Enable the module: `drush en views_string_aggregation`
3. Clear cache: `drush cr`

### Manual (UI)

1. Download and place the module in your `modules/custom/` directory
2. Enable the module at Admin > Modules

## Usage

### Basic Setup

1. Create or edit a View
2. Enable **"Use aggregation"** in the view's **Advanced** settings
3. Add the field you want to aggregate
4. Set the field's **Aggregation type** to **"String aggregation"** or **"String aggregation DISTINCT"**

### Configuration Options

#### Field Configuration
Set fields to use string aggregation by selecting **"String aggregation"** or **"String aggregation DISTINCT"** as the aggregation type in the field configuration.

#### Query Settings
When aggregation is enabled, additional setting options become available, under Views **"Advanced > Other > Query settings"**:

- **Concatenated values separator**: `SEPARATOR` character(s) to separate aggregated values (default: `,`)
- **Order concatenated values by**: Select an `ORDER BY` field to order concatenated values
- **Order direction**: Ascending or Descending sort order for concatenated values
- **Concatenated values maximum length**: Maximum length via group_concat_max_len (MySQL/MariaDB)

### Example Use Cases

#### 1. Node Tags Aggregation
Group nodes by category and show all tags as a comma-separated list:
```
Content Type | Tags
-------------|------------------
Article      | drupal, cms, web
Blog         | personal, tech
```

#### 2. User Roles Summary
Display all roles for users grouped by department:
```
Department | User Roles
-----------|------------------------
IT         | administrator, editor
Marketing  | editor, content creator
```

### Database-Specific Features

#### MySQL/MariaDB
- Uses `GROUP_CONCAT` for standard aggregation
- Uses `GROUP_CONCAT(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within `GROUP_CONCAT`
- Configurable separator and ordering
- Optionally override default max length via `group_concat_max_len`

#### PostgreSQL
- Uses `STRING_AGG` for standard aggregation
- Uses `STRING_AGG(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within aggregation function
- Configurable separator and ordering
- Note: order by field supports ASC/DESC direction
  but must use the aggregated field when using `DISTINCT` with pgsql

#### SQLite
- Uses `GROUP_CONCAT` for standard aggregation
- Uses `GROUP_CONCAT(DISTINCT` for distinct value aggregation
- Supports `ORDER BY` within `GROUP_CONCAT`
- Configurable separator and ordering
- Note: custom separator is not supported when using `DISTINCT` with SQLite

## API Reference

### Hook Implementations

- `hook_views_plugins_query_alter()`: Automatically extends the default Views query plugin
- `hook_config_schema_info_alter()`: Extends Views `views_query` schema with custom configuration options

### Query Plugin Classes

- `VsaBase`: Base class extending Drupal's SQL query plugin
- `MySql`: MySQL/MariaDB-specific implementation with `GROUP_CONCAT`
- `PgSql`: PostgreSQL-specific implementation with `STRING_AGG`
- `Sqlite`: SQLite-specific implementation with `GROUP_CONCAT`

### Filter Handlers

- `GroupByString`: Specialized filter for grouped string fields using `HAVING` clauses

### Contextual Filter Handlers

- `GroupByString`: Specialized contextual filter argument for grouped string fields using `HAVING` clauses

## Advanced Configuration

### Disable Automatic Query Plugin Extension
If you prefer to disable the automatic extension of the views_query plugin,
you can do so and then manually control which views use string aggregation:

1. Set `extend_views_query_plugin: false` in configuration
2. Clear cache
3. Manually assign Views String Aggregation query plugins to a table in `hook_views_data()`:

```php
// mysql
$views_data['table']['base']['query_id'] = 'vsa_views_query_mysql';
// postgres
$views_data['table']['base']['query_id'] = 'vsa_views_query_pgsql';
// or sqlite
$views_data['table']['base']['query_id'] = 'vsa_views_query_sqlite';
```

## Troubleshooting

**String aggregation option not appearing**
- Ensure "Use aggregation" is enabled in View advanced settings
- Verify the module is enabled and cache is cleared

### Performance Considerations

- String aggregation is performed at the SQL level for optimal performance
- Consider using `DISTINCT` aggregation when appropriate to reduce duplicate data
- Large result sets with many concatenated values may impact query performance

## Testing

The module includes automated kernel tests covering:
- Basic string aggregation functionality
- Distinct string aggregation functionality
- Separator configuration
- Exposed filter integration
- Sort order validation
- Database-specific implementations
- Maximum length configuration (MySQL/MariaDB)

Run tests with:
```bash
./vendor/bin/phpunit modules/custom/views_string_aggregation/tests/
```

## Credits

This module acknowledges with thanks the initial work contributed in
<https://www.drupal.org/project/drupal/issues/2902481> namely by:

<!-- cspell:disable -->

- f2boot
- phjou
- navkaur
- kapilv
- jackson.cooper
- attisan
- simeonkesmev
- achton
- kasperg
- Sir-Arturio
- b.ravanbakhsh

<!-- cspell:enable -->

## Contributing

This module follows Drupal coding standards and best practices. Contributions are welcome for:
- Additional database engine support
- Performance optimizations
- Enhanced configuration options
- Bug fixes and improvements

## License

This project is licensed under the GPL-2.0+ license.

## Support

For issues, feature requests, or questions:
1. Check existing issues in the project repository
2. Review the troubleshooting section above
3. Create a detailed issue report with:
   - Drupal version
   - Database type and version
   - Steps to reproduce
   - Expected vs actual behavior
